@@ -42,26 +42,10 @@ static int __init graphicsmaster_init(void)
 	/*
 	 * Probe for SA1111.
 	 */
-	ret = sa1111_probe(0x18000000);
+	ret = sa1111_init(NULL, 0x18000000, ADS_EXT_IRQ(0));
 	if (ret < 0)
 		return ret;
 
-	/*
-	 * We found it.  Wake the chip up.
-	 */
-	sa1111_wake();
-
-	/*
-	 * The SDRAM configuration of the SA1110 and the SA1111 must
-	 * match.  This is very important to ensure that SA1111 accesses
-	 * don't corrupt the SDRAM.  Note that this ungates the SA1111's
-	 * MBGNT signal, so we must have called sa1110_mb_disable()
-	 * beforehand.
-	 */
-	sa1111_configure_smc(1,
-			     FExtr(MDCNFG, MDCNFG_SA1110_DRAC0),
-			     FExtr(MDCNFG, MDCNFG_SA1110_TDL0));
-
 	/*
 	 * Enable PWM control for LCD
 	 */
@@ -71,19 +55,6 @@ static int __init graphicsmaster_init(void)
 	SKPWM1 = 0x01;				// Backlight
 	SKPEN1 = 1;
 
-	/*
-	 * We only need to turn on DCLK whenever we want to use the
-	 * DMA.  It can otherwise be held firmly in the off position.
-	 */
-	SKPCR |= SKPCR_DCLKEN;
-
-	/*
-	 * Enable the SA1110 memory bus request and grant signals.
-	 */
-	sa1110_mb_enable();
-
-	sa1111_init_irq(ADS_EXT_IRQ(0));
-
 	return 0;
 }
 
@@ -186,7 +157,7 @@ static void __init graphicsmaster_init_irq(void)
  */
 
 static void __init
-fixup_graphicsmaster(struct machine_desc *desc, struct param_struct *params,
+fixup_graphicsmaster(struct machine_desc *desc, struct tag *tags,
 		     char **cmdline, struct meminfo *mi)
 {
 	SET_BANK( 0, 0xc0000000, 16*1024*1024 );
