@@ -1667,6 +1667,7 @@ static int apm(void *unused)
 	daemonize();
 
 	strcpy(current->comm, "kapmd");
+	current->flags |= PF_IOTHREAD;
 	sigfillset(&current->blocked);
 
 	if (apm_info.connection_version == 0) {
@@ -2024,5 +2025,3 @@ MODULE_PARM_DESC(idle_threshold,
 MODULE_PARM(idle_period, "i");
 MODULE_PARM_DESC(idle_period,
 	"Period (in sec/100) over which to caculate the idle percentage");
-
-EXPORT_NO_SYMBOLS;
