@@ -21,6 +21,7 @@
 
 #include <asm/mtrr.h>
 #include <asm/pgalloc.h>
+#include <asm/tlbflush.h>
 #include <asm/smpboot.h>
 
 /*
@@ -66,7 +67,7 @@
  *	an L1cache=Writethrough or L1cache=off option.
  *
  *		B stepping CPUs may hang. There are hardware work arounds
- *	for this. We warn about it in case your board doesn't have the work
+ *	for this. We warn about it in case your board doesnt have the work
  *	arounds. Basically thats so I can tell anyone with a B stepping
  *	CPU and SMP problems "tough".
  *
@@ -103,7 +104,7 @@
  */
 
 /* The 'big kernel lock' */
-spinlock_cacheline_t kernel_flag_cacheline = {SPIN_LOCK_UNLOCKED};
+spinlock_t kernel_flag __cacheline_aligned_in_smp = SPIN_LOCK_UNLOCKED;
 
 struct tlb_state cpu_tlbstate[NR_CPUS] __cacheline_aligned = {[0 ... NR_CPUS-1] = { &init_mm, 0, }};
 
@@ -115,7 +116,7 @@ struct tlb_state cpu_tlbstate[NR_CPUS] __cacheline_aligned = {[0 ... NR_CPUS-1]
 
 static inline int __prepare_ICR (unsigned int shortcut, int vector)
 {
-	return APIC_DM_FIXED | shortcut | vector | INT_DEST_ADDR_MODE;
+	return APIC_DM_FIXED | shortcut | vector | APIC_DEST_LOGICAL;
 }
 
 static inline int __prepare_ICR2 (unsigned int mask)
@@ -300,11 +301,12 @@ static spinlock_t tlbstate_lock = SPIN_LOCK_UNLOCKED;
  * instead update mm->cpu_vm_mask.
  *
  * We need to reload %cr3 since the page tables may be going
- * away frm under us...
+ * away from under us..
  */
 static void inline leave_mm (unsigned long cpu)
 {
-	BUG_ON(cpu_tlbstate[cpu].state == TLBSTATE_OK);
+	if (cpu_tlbstate[cpu].state == TLBSTATE_OK)
+		BUG();
 	clear_bit(cpu, &cpu_tlbstate[cpu].active_mm->cpu_vm_mask);
 	load_cr3(swapper_pg_dir);
 }
@@ -493,12 +495,22 @@ void flush_tlb_all(void)
  * it goes straight through and wastes no time serializing
  * anything. Worst case is that we lose a reschedule ...
  */
-
 void smp_send_reschedule(int cpu)
 {
 	send_IPI_mask(1 << cpu, RESCHEDULE_VECTOR);
 }
 
+/*
+ * this function sends a reschedule IPI to all (other) CPUs.
+ * This should only be used if some 'global' task became runnable,
+ * such as a RT task, that must be handled now. The first CPU
+ * that manages to grab the task will run it.
+ */
+void smp_send_reschedule_all(void)
+{
+	send_IPI_allbutself(RESCHEDULE_VECTOR);
+}
+
 /*
  * Structure and data for smp_call_function(). This is designed to minimise
  * static memory requirements. It also looks cleaner.
