@@ -71,7 +71,6 @@ static int debug_close(struct inode *inode, struct file *file);
 static struct proc_dir_entry 
 *debug_create_proc_dir_entry(struct proc_dir_entry *root,
 			     const char *name, mode_t mode,
-			     struct inode_operations *iops,
 			     struct file_operations *fops);
 static void debug_delete_proc_dir_entry(struct proc_dir_entry *root,
 					struct proc_dir_entry *entry);
@@ -166,13 +165,6 @@ static struct file_operations debug_file_ops = {
 	release: debug_close,
 };
 
-static struct inode_operations debug_inode_ops = {
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,98))
-	default_file_ops: &debug_file_ops,	/* file ops */
-#endif
-};
-
-
 static struct proc_dir_entry *debug_proc_root_entry;
 
 /* functions */
@@ -228,10 +220,8 @@ static debug_info_t*  debug_info_alloc(char *name, int page_order,
 	strncpy(rc->name, name, MIN(strlen(name), (DEBUG_MAX_PROCF_LEN - 1)));
 	rc->name[MIN(strlen(name), (DEBUG_MAX_PROCF_LEN - 1))] = 0;
 	memset(rc->views, 0, DEBUG_MAX_VIEWS * sizeof(struct debug_view *));
-#ifdef CONFIG_PROC_FS
 	memset(rc->proc_entries, 0 ,DEBUG_MAX_VIEWS *
 		sizeof(struct proc_dir_entry*));
-#endif /* CONFIG_PROC_FS */
 	atomic_set(&(rc->ref_count), 0);
 
 	return rc;
@@ -279,11 +269,7 @@ static debug_info_t*  debug_info_create(char *name, int page_order,
 
 	/* create proc rood directory */
 
-        rc->proc_root_entry =
-            debug_create_proc_dir_entry(debug_proc_root_entry, rc->name,
-                                        S_IFDIR | S_IRUGO | S_IXUGO |
-                                        S_IWUSR | S_IWGRP, NULL, NULL);
-
+        rc->proc_root_entry = proc_mkdir(rc->name, debug_proc_root_entry);
 	/* append new element to linked list */
 
         if(debug_area_first == NULL){
@@ -348,10 +334,8 @@ static void debug_info_put(debug_info_t *db_info)
 	if (!db_info)
 		return;
 	if (atomic_dec_and_test(&db_info->ref_count)) {
-#ifdef DEBUG
 		printk(KERN_INFO "debug: freeing debug area %p (%s)\n",
 		       db_info, db_info->name);
-#endif
 		for (i = 0; i < DEBUG_MAX_VIEWS; i++) {
 			if (db_info->views[i] != NULL)
 				debug_delete_proc_dir_entry
@@ -459,9 +443,9 @@ static ssize_t debug_output(struct file *file,	/* file descriptor */
 		size = MIN((len - count), (size - entry_offset));
 
 		if(size){
-			if (copy_to_user(user_buf + count, 
-					p_info->temp_buf + entry_offset, size))
-				return -EFAULT;
+			if ((rc = copy_to_user(user_buf + count, 
+					p_info->temp_buf + entry_offset, size)))
+			return rc;
 		}
 		count += size;
 		entry_offset = 0;
@@ -527,8 +511,8 @@ static int debug_open(struct inode *inode, struct file *file)
 		for (i = 0; i < DEBUG_MAX_VIEWS; i++) {
 			if (debug_info->views[i] == NULL)
 				continue;
-			else if (debug_info->proc_entries[i]->low_ino ==
-				 file->f_dentry->d_inode->i_ino) {
+			else if (debug_info->proc_entries[i] ==
+				 PDE(file->f_dentry->d_inode)) {
 				goto found;	/* found view ! */
 			}
 		}
@@ -545,18 +529,14 @@ static int debug_open(struct inode *inode, struct file *file)
 	debug_info_snapshot = debug_info_copy(debug_info);
 
 	if(!debug_info_snapshot){
-#ifdef DEBUG
 		printk(KERN_ERR "debug_open: debug_info_copy failed (out of mem)\n");
-#endif
 		rc = -ENOMEM;
 		goto out;
 	}
 
 	if ((file->private_data =
 	     kmalloc(sizeof(file_private_info_t), GFP_ATOMIC)) == 0) {
-#ifdef DEBUG
 		printk(KERN_ERR "debug_open: kmalloc failed\n");
-#endif
 		debug_info_free(debug_info_snapshot);	
 		rc = -ENOMEM;
 		goto out;
@@ -606,44 +586,11 @@ static int debug_close(struct inode *inode, struct file *file)
 
 static struct proc_dir_entry *debug_create_proc_dir_entry
     (struct proc_dir_entry *root, const char *name, mode_t mode,
-     struct inode_operations *iops, struct file_operations *fops) 
+     struct file_operations *fops) 
 {
-	struct proc_dir_entry *rc = NULL;
-
-#ifdef CONFIG_PROC_FS
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,98))
-	const char *fn = name;
-	int len;
-	len = strlen(fn);
-
-	rc = (struct proc_dir_entry *) kmalloc(sizeof(struct proc_dir_entry)
-					       + len + 1, GFP_ATOMIC);
-	if (!rc)
-		goto out;
-
-	memset(rc, 0, sizeof(struct proc_dir_entry));
-	memcpy(((char *) rc) + sizeof(*rc), fn, len + 1);
-	rc->name = ((char *) rc) + sizeof(*rc);
-	rc->namelen = len;
-	rc->low_ino = 0, rc->mode = mode;
-	rc->nlink = 1;
-	rc->uid = 0;
-	rc->gid = 0;
-	rc->size = 0;
-	rc->get_info = NULL;
-	rc->ops = iops;
-
-	proc_register(root, rc);
-#else
-	rc = create_proc_entry(name, mode, root);
-	if (!rc)
-		goto out;
-	if (fops)
+	struct proc_dir_entry *rc = create_proc_entry(name, mode, root);
+	if (rc && fops)
 		rc->proc_fops = fops;
-#endif
-
-      out:
-#endif /* CONFIG_PROC_FS */
 	return rc;
 }
 
@@ -655,15 +602,7 @@ static struct proc_dir_entry *debug_create_proc_dir_entry
 static void debug_delete_proc_dir_entry
     (struct proc_dir_entry *root, struct proc_dir_entry *proc_entry) 
 {
-
-#ifdef CONFIG_PROC_FS
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,98))
-	proc_unregister(root, proc_entry->low_ino);
-	kfree(proc_entry);
-#else
 	remove_proc_entry(proc_entry->name, root);
-#endif
-#endif /* CONFIG_PROC_FS */
 }
 
 /*
@@ -689,11 +628,9 @@ debug_info_t *debug_register
 		goto out;
 	debug_register_view(rc, &debug_level_view);
         debug_register_view(rc, &debug_flush_view);
-#ifdef DEBUG
 	printk(KERN_INFO
 	       "debug: reserved %d areas of %d pages for debugging %s\n",
 	       nr_areas, 1 << page_order, rc->name);
-#endif
       out:
         if (rc == NULL){
 		printk(KERN_ERR "debug: debug_register failed for %s\n",name);
@@ -713,9 +650,7 @@ void debug_unregister(debug_info_t * id)
 	if (!id)
 		goto out;
 	down(&debug_lock);
-#ifdef DEBUG
 	printk(KERN_INFO "debug: unregistering %s\n", id->name);
-#endif
 	debug_info_put(id);
 	up(&debug_lock);
 
@@ -744,10 +679,8 @@ void debug_set_level(debug_info_t* id, int new_level)
                         id->name, new_level, 0, DEBUG_MAX_LEVEL);
         } else {
                 id->level = new_level;
-#ifdef DEBUG
                 printk(KERN_INFO 
 			"debug: %s: new level %i\n",id->name,id->level);
-#endif
         }
 	spin_unlock_irqrestore(&id->lock,flags);
 }
@@ -924,13 +857,7 @@ int debug_init(void)
 
 	down(&debug_lock);
 	if (!initialized) {
-#ifdef CONFIG_PROC_FS
-		debug_proc_root_entry =
-		    debug_create_proc_dir_entry(&proc_root, DEBUG_DIR_ROOT,
-						S_IFDIR | S_IRUGO | S_IXUGO
-						| S_IWUSR | S_IWGRP, NULL,
-						NULL);
-#endif /* CONFIG_PROC_FS */
+		debug_proc_root_entry = proc_mkdir(DEBUG_DIR_ROOT, NULL);
 		printk(KERN_INFO "debug: Initialization complete\n");
 		initialized = 1;
 	}
@@ -949,21 +876,9 @@ int debug_register_view(debug_info_t * id, struct debug_view *view)
 	int i;
 	unsigned long flags;
 	mode_t mode = S_IFREG;
-	struct proc_dir_entry *pde;
-
 
 	if (!id)
 		goto out;
-	pde = debug_create_proc_dir_entry(id->proc_root_entry,
-					  view->name, mode,
-					  &debug_inode_ops,
-					  &debug_file_ops);
-	if(!pde){
-		printk(KERN_WARNING "debug: create_proc_entry() failed! Cannot register view %s/%s\n", id->name,view->name);
-		rc = -1;
-		goto out;
-	}
-
 	spin_lock_irqsave(&id->lock, flags);
 	for (i = 0; i < DEBUG_MAX_VIEWS; i++) {
 		if (id->views[i] == NULL)
@@ -974,8 +889,6 @@ int debug_register_view(debug_info_t * id, struct debug_view *view)
 			id->name,view->name);
 		printk(KERN_WARNING 
 			"debug: maximum number of views reached (%i)!\n", i);
-		debug_delete_proc_dir_entry(id->proc_root_entry, pde);
-		
 		rc = -1;
 	}
 	else {
@@ -984,7 +897,10 @@ int debug_register_view(debug_info_t * id, struct debug_view *view)
 			mode |= S_IRUSR;
 		if (view->input_proc)
 			mode |= S_IWUSR;
-		id->proc_entries[i] = pde;
+		id->proc_entries[i] =
+		    debug_create_proc_dir_entry(id->proc_root_entry,
+						view->name, mode,
+						&debug_file_ops);
 		rc = 0;
 	}
 	spin_unlock_irqrestore(&id->lock, flags);
@@ -1301,9 +1217,7 @@ void cleanup_module(void)
 #ifdef DEBUG
 	printk("debug_cleanup_module: \n");
 #endif
-#ifdef CONFIG_PROC_FS
 	debug_delete_proc_dir_entry(&proc_root, debug_proc_root_entry);
-#endif /* CONFIG_PROC_FS */
 	return;
 }
 
