@@ -3,6 +3,13 @@
 #include <linux/linkage.h>
 #include <linux/sys.h>
 #include <linux/cache.h>
+#include <linux/config.h>
+
+/* No comment. */
+#if defined(CONFIG_NFSD) || defined(CONFIG_NFSD_MODULE)
+#else
+#define sys_nfsservctl		sys_ni_syscall
+#endif
 
 #define __NO_STUBS
 
