@@ -23,27 +23,16 @@
  * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * RED HAT AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
- * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
- * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *
- * THIS SOFTWARE IS NOT INTENDED FOR USE IN SAFETY CRITICAL SYSTEMS
- *
  * Authors:
  *    Gareth Hughes <gareth@valinux.com>
- *
- * Memory allocation size checks added 14/01/2003, Alan Cox <alan@redhat.com>
  */
 
+#define __NO_VERSION__
 #include "r128.h"
 #include "drmP.h"
-#include "drm.h"
-#include "r128_drm.h"
 #include "r128_drv.h"
+#include "drm.h"
+#include <linux/delay.h>
 
 
 /* ================================================================
@@ -540,7 +529,7 @@ static void r128_cce_dispatch_flip( drm_device_t *dev )
 {
 	drm_r128_private_t *dev_priv = dev->dev_private;
 	RING_LOCALS;
-	DRM_DEBUG( "page=%d\n", dev_priv->current_page );
+	DRM_DEBUG( "%s: page=%d\n", __FUNCTION__, dev_priv->current_page );
 
 #if R128_PERFORMANCE_BOXES
 	/* Do some trivial performance monitoring...
@@ -589,7 +578,8 @@ static void r128_cce_dispatch_vertex( drm_device_t *dev,
 	int prim = buf_priv->prim;
 	int i = 0;
 	RING_LOCALS;
-	DRM_DEBUG( "buf=%d nbox=%d\n", buf->idx, sarea_priv->nbox );
+	DRM_DEBUG( "%s: buf=%d nbox=%d\n",
+		   __FUNCTION__, buf->idx, sarea_priv->nbox );
 
 	if ( 0 )
 		r128_print_dirty( "dispatch_vertex", sarea_priv->dirty );
@@ -800,7 +790,7 @@ static int r128_cce_dispatch_blit( drm_device_t *dev,
 	u32 *data;
 	int dword_shift, dwords;
 	RING_LOCALS;
-	DRM_DEBUG( "\n" );
+	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	/* The compiler won't optimize away a division by a variable,
 	 * even if the only legal values are powers of two.  Thus, we'll
@@ -913,9 +903,6 @@ static int r128_cce_dispatch_write_span( drm_device_t *dev,
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	count = depth->n;
-	
-	if( count > 4096 || count <= 0)
-		return -EMSGSIZE;
 	if ( copy_from_user( &x, depth->x, sizeof(x) ) ) {
 		return -EFAULT;
 	}
@@ -1009,9 +996,6 @@ static int r128_cce_dispatch_write_pixels( drm_device_t *dev,
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	count = depth->n;
-	
-	if( count > 4096 || count <= 0)
-		return -EMSGSIZE;
 
 	x = kmalloc( count * sizeof(*x), GFP_KERNEL );
 	if ( x == NULL ) {
@@ -1127,9 +1111,6 @@ static int r128_cce_dispatch_read_span( drm_device_t *dev,
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	count = depth->n;
-	
-	if ( count > 4096 || count <= 0)
-		return -EMSGSIZE;
 	if ( copy_from_user( &x, depth->x, sizeof(x) ) ) {
 		return -EFAULT;
 	}
@@ -1172,9 +1153,6 @@ static int r128_cce_dispatch_read_pixels( drm_device_t *dev,
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	count = depth->n;
-	if ( count > 4096 || count <= 0)
-		return -EMSGSIZE;
-
 	if ( count > dev_priv->depth_pitch ) {
 		count = dev_priv->depth_pitch;
 	}
@@ -1262,22 +1240,18 @@ int r128_cce_clear( struct inode *inode, struct file *filp,
 	drm_file_t *priv = filp->private_data;
 	drm_device_t *dev = priv->dev;
 	drm_r128_private_t *dev_priv = dev->dev_private;
-	drm_r128_sarea_t *sarea_priv;
+	drm_r128_sarea_t *sarea_priv = dev_priv->sarea_priv;
 	drm_r128_clear_t clear;
 	DRM_DEBUG( "%s\n", __FUNCTION__ );
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
-
 	if ( copy_from_user( &clear, (drm_r128_clear_t *) arg,
 			     sizeof(clear) ) )
 		return -EFAULT;
 
 	RING_SPACE_TEST_WITH_RETURN( dev_priv );
 
-	sarea_priv = dev_priv->sarea_priv;
-
 	if ( sarea_priv->nbox > R128_NR_SAREA_CLIPRECTS )
 		sarea_priv->nbox = R128_NR_SAREA_CLIPRECTS;
 
@@ -1301,8 +1275,6 @@ int r128_cce_swap( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
-
 	RING_SPACE_TEST_WITH_RETURN( dev_priv );
 
 	if ( sarea_priv->nbox > R128_NR_SAREA_CLIPRECTS )
@@ -1332,7 +1304,10 @@ int r128_cce_vertex( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
+	if ( !dev_priv ) {
+		DRM_ERROR( "%s called with no initialization\n", __FUNCTION__ );
+		return -EINVAL;
+	}
 
 	if ( copy_from_user( &vertex, (drm_r128_vertex_t *)arg,
 			     sizeof(vertex) ) )
@@ -1392,7 +1367,10 @@ int r128_cce_indices( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
+	if ( !dev_priv ) {
+		DRM_ERROR( "%s called with no initialization\n", __FUNCTION__ );
+		return -EINVAL;
+	}
 
 	if ( copy_from_user( &elts, (drm_r128_indices_t *)arg,
 			     sizeof(elts) ) )
@@ -1461,8 +1439,6 @@ int r128_cce_blit( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
-
 	if ( copy_from_user( &blit, (drm_r128_blit_t *)arg,
 			     sizeof(blit) ) )
 		return -EFAULT;
@@ -1523,8 +1499,6 @@ int r128_cce_stipple( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
-
 	if ( copy_from_user( &stipple, (drm_r128_stipple_t *)arg,
 			     sizeof(stipple) ) )
 		return -EFAULT;
@@ -1556,7 +1530,10 @@ int r128_cce_indirect( struct inode *inode, struct file *filp,
 
 	LOCK_TEST_WITH_RETURN( dev );
 
-	DEV_INIT_TEST_WITH_RETURN(dev_priv);
+	if ( !dev_priv ) {
+		DRM_ERROR( "%s called with no initialization\n", __FUNCTION__ );
+		return -EINVAL;
+	}
 
 	if ( copy_from_user( &indirect, (drm_r128_indirect_t *)arg,
 			     sizeof(indirect) ) )
