@@ -1,4 +1,4 @@
-/* $Id: hysdn_proclog.c,v 1.1.4.1 2001/11/20 14:19:37 kai Exp $
+/* $Id: hysdn_proclog.c,v 1.9.6.3 2001/09/23 22:24:54 kai Exp $
  *
  * Linux driver for HYSDN cards, /proc/net filesystem log functions.
  *
@@ -207,21 +207,19 @@ hysdn_log_read(struct file *file, char *buf, size_t count, loff_t * off)
 {
 	struct log_data *inf;
 	int len;
-	word ino;
+	struct proc_dir_entry *pde = PDE(file->f_dentry->d_inode);
 	struct procdata *pd = NULL;
 	hysdn_card *card;
-	loff_t pos = *off;
 
 	if (!*((struct log_data **) file->private_data)) {
 		if (file->f_flags & O_NONBLOCK)
 			return (-EAGAIN);
 
 		/* sorry, but we need to search the card */
-		ino = file->f_dentry->d_inode->i_ino & 0xFFFF;	/* low-ino */
 		card = card_root;
 		while (card) {
 			pd = card->proclog;
-			if (pd->log->low_ino == ino)
+			if (pd->log == pde)
 				break;
 			card = card->next;	/* search next entry */
 		}
@@ -239,7 +237,7 @@ hysdn_log_read(struct file *file, char *buf, size_t count, loff_t * off)
 	if ((len = strlen(inf->log_start)) <= count) {
 		if (copy_to_user(buf, inf->log_start, len))
 			return -EFAULT;
-		*off = pos + len;
+		file->f_pos += len;
 		return (len);
 	}
 	return (0);
@@ -259,7 +257,7 @@ hysdn_log_open(struct inode *ino, struct file *filep)
 	card = card_root;
 	while (card) {
 		pd = card->proclog;
-		if (pd->log->low_ino == (ino->i_ino & 0xFFFF))
+		if (pd->log == PDE(ino))
 			break;
 		card = card->next;	/* search next entry */
 	}
@@ -324,7 +322,7 @@ hysdn_log_close(struct inode *ino, struct file *filep)
 			card = card_root;
 			while (card) {
 				pd = card->proclog;
-				if (pd->log->low_ino == (ino->i_ino & 0xFFFF))
+				if (pd->log == PDE(ino))
 					break;
 				card = card->next;	/* search next entry */
 			}
@@ -360,7 +358,7 @@ static unsigned int
 hysdn_log_poll(struct file *file, poll_table * wait)
 {
 	unsigned int mask = 0;
-	word ino;
+	struct proc_dir_entry *pde = PDE(file->f_dentry->d_inode);
 	hysdn_card *card;
 	struct procdata *pd = NULL;
 
@@ -368,11 +366,10 @@ hysdn_log_poll(struct file *file, poll_table * wait)
 		return (mask);	/* no polling for write supported */
 
 	/* we need to search the card */
-	ino = file->f_dentry->d_inode->i_ino & 0xFFFF;	/* low-ino */
 	card = card_root;
 	while (card) {
 		pd = card->proclog;
-		if (pd->log->low_ino == ino)
+		if (pd->log == pde)
 			break;
 		card = card->next;	/* search next entry */
 	}
