@@ -676,6 +676,7 @@ i596_rx_one(struct net_device *dev, volatile struct i596_private *lp,
 
 		skb->protocol = eth_type_trans(skb,dev);
 		netif_rx(skb);
+		dev->last_rx = jiffies;
 		lp->stats.rx_packets++;
 	} else {
 #if 0
@@ -886,16 +887,7 @@ i596_start_xmit (struct sk_buff *skb, struct net_device *dev) {
 	if (skb->len <= 0)
 		return 0;
 
-	length = skb->len;
-	
-	if(length < ETH_ZLEN)
-	{
-		skb = skb_padto(skb, ETH_ZLEN);
-		if(skb == NULL)
-			return 0;
-		length = ETH_ZLEN;
-	}
-	
+	length = (ETH_ZLEN < skb->len) ? skb->len : ETH_ZLEN;
 	dev->trans_start = jiffies;
 
 	tx_cmd = (struct tx_cmd *)
