@@ -189,8 +189,6 @@ MODULE_PARM_DESC(sizes, "list of device (partition) sizes " \
 /* The following items are obtained through kmalloc() in init_module() */
 
 Xpram_Dev *xpram_devices = NULL;
-int *xpram_blksizes = NULL;
-int *xpram_hardsects = NULL;
 int *xpram_offsets = NULL;   /* partition offsets */
 
 #define MIN(x,y) ((x) < (y) ? (x) : (y))
@@ -749,7 +747,10 @@ void xpram_request(request_queue_t * queue)
 #endif /* V24 */
 
 	while(1) {
-		INIT_REQUEST;
+		if (blk_queue_empty(QUEUE)) {
+			CLEAR_INTR;
+			return;
+		}
 
 		fault=0;
 #if ( XPRAM_VERSION == 24 )
@@ -1022,6 +1023,7 @@ int xpram_init(void)
 #elif (XPRAM_VERSION == 24)
 	q = BLK_DEFAULT_QUEUE (major);
 	blk_init_queue (q, xpram_request);
+	blk_queue_hardsect_size(q, xpram_hardsect);
 #endif /* V22/V24 */
 
 	/* we want to have XPRAM_UNUSED blocks security buffer between devices */
@@ -1062,26 +1064,6 @@ int xpram_init(void)
 		PRINT_DEBUG(" device(%d) offset = %d kB, size = %d kB\n",i, xpram_offsets[i], xpram_sizes[i]);
 #endif
 
-	xpram_blksizes = kmalloc(xpram_devs * sizeof(int), GFP_KERNEL);
-	if (!xpram_blksizes) {
-		PRINT_ERR("Not enough memory for xpram_blksizes\n");
-                PRINT_ERR("Giving up xpram\n");
-		goto fail_malloc_blksizes;
-	}
-	for (i=0; i < xpram_devs; i++) /* all the same blocksize */
-		xpram_blksizes[i] = xpram_blksize;
-	blksize_size[major]=xpram_blksizes;
-
-	xpram_hardsects = kmalloc(xpram_devs * sizeof(int), GFP_KERNEL);
-	if (!xpram_hardsects) {
-		PRINT_ERR("Not enough memory for xpram_hardsects\n");
-                PRINT_ERR("Giving up xpram\n");
-		goto fail_malloc_hardsects;
-	}
-	for (i=0; i < xpram_devs; i++) /* all the same hardsect */
-		xpram_hardsects[i] = xpram_hardsect;
-	hardsect_size[major]=xpram_hardsects;
-   
 	/* 
 	 * allocate the devices -- we can't have them static, as the number
 	 * can be specified at load time
@@ -1152,14 +1134,8 @@ int xpram_init(void)
 	}
 	kfree(xpram_devices);
 #endif /* V24 */
- fail_malloc_blksizes:
 	kfree (xpram_offsets);
- fail_malloc_hardsects:
-	kfree (xpram_blksizes);
-	blksize_size[major] = NULL;
  fail_malloc_devices:
-	kfree(xpram_hardsects);
-	hardsect_size[major] = NULL;
  fail_malloc:
 #if (XPRAM_VERSION == 22)
 	blk_dev[major].request_fn = NULL;
@@ -1196,8 +1172,6 @@ void cleanup_module(void)
 #if (XPRAM_VERSION == 22)
 	blk_dev[major].request_fn = NULL;
 #endif /* V22 */
-	kfree(blksize_size[major]);
-	kfree(hardsect_size[major]);
 	kfree(xpram_offsets);
 	blk_clear(major);
 
