@@ -5,10 +5,9 @@
  */
 
 #include <linux/fs.h>
-#include <linux/locks.h>
-#include <linux/bfs_fs.h>
+#include <linux/buffer_head.h>
 #include <linux/smp_lock.h>
-#include "bfs_defs.h"
+#include "bfs.h"
 
 #undef DEBUG
 
