@@ -9,17 +9,15 @@
 #include <linux/mm.h>
 #include <linux/slab.h>
 #include <linux/init.h>
-#include <linux/bfs_fs.h>
+#include <linux/fs.h>
 #include <linux/smp_lock.h>
-
+#include <linux/buffer_head.h>
 #include <asm/uaccess.h>
-
-#include "bfs_defs.h"
+#include "bfs.h"
 
 MODULE_AUTHOR("Tigran A. Aivazian <tigran@veritas.com>");
 MODULE_DESCRIPTION("SCO UnixWare BFS filesystem for Linux");
 MODULE_LICENSE("GPL");
-EXPORT_NO_SYMBOLS;
 
 #undef DEBUG
 
