@@ -12,11 +12,12 @@
 
 #include <linux/module.h>
 #include <linux/time.h>
-#include <linux/locks.h>
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
 #include <linux/msdos_fs.h>
 #include <linux/fat_cvf.h>
+#include <linux/pagemap.h>
+#include <linux/buffer_head.h>
 
 //#include <asm/uaccess.h>
 #include <asm/unaligned.h>
@@ -120,7 +121,7 @@ struct inode *fat_iget(struct super_block *sb, int i_pos)
 	return inode;
 }
 
-static void fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de);
+static int fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de);
 
 struct inode *fat_build_inode(struct super_block *sb,
 				struct msdos_dir_entry *de, int ino, int *res)
@@ -134,10 +135,14 @@ struct inode *fat_build_inode(struct super_block *sb,
 	*res = -ENOMEM;
 	if (!inode)
 		goto out;
-	*res = 0;
 	inode->i_ino = iunique(sb, MSDOS_ROOT_INO);
 	inode->i_version = 0;
-	fat_fill_inode(inode, de);
+	*res = fat_fill_inode(inode, de);
+	if (*res < 0) {
+		iput(inode);
+		inode = NULL;
+		goto out;
+	}
 	fat_attach(inode, ino);
 	insert_inode_hash(inode);
 out:
@@ -173,9 +178,7 @@ void fat_put_super(struct super_block *sb)
 		dec_cvf_format_use_count_by_version(sbi->cvf_format->cvf_version);
 		sbi->cvf_format->unmount_cvf(sb);
 	}
-	if (sbi->fat_bits == 32) {
-		fat_clusters_flush(sb);
-	}
+	fat_clusters_flush(sb);
 	fat_cache_inval_dev(sb);
 	if (sbi->nls_disk) {
 		unload_nls(sbi->nls_disk);
@@ -357,37 +360,43 @@ out:
 	return ret;
 }
 
-static void fat_calc_dir_size(struct inode *inode)
+static int fat_calc_dir_size(struct inode *inode)
 {
 	struct super_block *sb = inode->i_sb;
 	int nr;
 
 	inode->i_size = 0;
 	if (MSDOS_I(inode)->i_start == 0)
-		return;
+		return 0;
 
 	nr = MSDOS_I(inode)->i_start;
 	do {
 		inode->i_size += 1 << MSDOS_SB(sb)->cluster_bits;
-		if (!(nr = fat_access(sb, nr, -1))) {
-			printk("FAT: Directory %ld: bad FAT\n",
-			       inode->i_ino);
-			break;
+		nr = fat_access(sb, nr, -1);
+		if (nr < 0)
+			return nr;
+		else if (nr == FAT_ENT_FREE) {
+			fat_fs_panic(sb, "Directory %lu: invalid cluster chain",
+				     inode->i_ino);
+			return -EIO;
 		}
 		if (inode->i_size > FAT_MAX_DIR_SIZE) {
-			fat_fs_panic(sb, "Directory %ld: "
+			fat_fs_panic(sb, "Directory %lu: "
 				     "exceeded the maximum size of directory",
 				     inode->i_ino);
 			inode->i_size = FAT_MAX_DIR_SIZE;
 			break;
 		}
-	} while (nr != -1);
+	} while (nr != FAT_ENT_EOF);
+
+	return 0;
 }
 
-static void fat_read_root(struct inode *inode)
+static int fat_read_root(struct inode *inode)
 {
 	struct super_block *sb = inode->i_sb;
 	struct msdos_sb_info *sbi = MSDOS_SB(sb);
+	int error;
 
 	MSDOS_I(inode)->i_location = 0;
 	inode->i_uid = sbi->options.fs_uid;
@@ -399,7 +408,9 @@ static void fat_read_root(struct inode *inode)
 	inode->i_fop = &fat_dir_operations;
 	if (sbi->fat_bits == 32) {
 		MSDOS_I(inode)->i_start = sbi->root_cluster;
-		fat_calc_dir_size(inode);
+		error = fat_calc_dir_size(inode);
+		if (error < 0)
+			return error;
 	} else {
 		MSDOS_I(inode)->i_start = 0;
 		inode->i_size = sbi->dir_entries * sizeof(struct msdos_dir_entry);
@@ -414,6 +425,8 @@ static void fat_read_root(struct inode *inode)
 	inode->i_mtime = inode->i_atime = inode->i_ctime = 0;
 	MSDOS_I(inode)->i_ctime_ms = 0;
 	inode->i_nlink = fat_subdirs(inode)+2;
+
+	return 0;
 }
 
 /*
@@ -426,21 +439,26 @@ static void fat_read_root(struct inode *inode)
  *  4/  parent->i_logstart - maybe used to hunt for the file on disc
  *
  */
-struct dentry *fat_fh_to_dentry(struct super_block *sb, __u32 *fh,
-				int len, int fhtype, int parent)
+
+struct dentry *fat_decode_fh(struct super_block *sb, __u32 *fh,
+			     int len, int fhtype, 
+			     int (*acceptable)(void *context, struct dentry *de),
+			     void *context)
 {
-	struct inode *inode = NULL;
-	struct list_head *lp;
-	struct dentry *result;
 
 	if (fhtype != 3)
 		return ERR_PTR(-ESTALE);
 	if (len < 5)
 		return ERR_PTR(-ESTALE);
-	/* We cannot find the parent,
-	   It better just *be* there */
-	if (parent)
-		return ERR_PTR(-ESTALE);
+
+	return sb->s_export_op->find_exported_dentry(sb, fh, NULL, acceptable, context);
+}
+
+struct dentry *fat_get_dentry(struct super_block *sb, void *inump)
+{
+	struct inode *inode = NULL;
+	struct dentry *result;
+	__u32 *fh = inump;
 
 	inode = iget(sb, fh[0]);
 	if (!inode || is_bad_inode(inode) ||
@@ -480,35 +498,19 @@ struct dentry *fat_fh_to_dentry(struct super_block *sb, __u32 *fh,
 	
 	/* now to find a dentry.
 	 * If possible, get a well-connected one
-	 *
-	 * Given the way that we found the inode, it *MUST* be
-	 * well-connected, but it is easiest to just copy the
-	 * code.
 	 */
-	spin_lock(&dcache_lock);
-	list_for_each(lp, &inode->i_dentry) {
-		result = list_entry(lp,struct dentry, d_alias);
-		if (! (result->d_flags & DCACHE_NFSD_DISCONNECTED)) {
-			dget_locked(result);
-			result->d_vfs_flags |= DCACHE_REFERENCED;
-			spin_unlock(&dcache_lock);
-			iput(inode);
-			return result;
-		}
-	}
-	spin_unlock(&dcache_lock);
-	result = d_alloc_root(inode);
+	result = d_alloc_anon(inode);
 	if (result == NULL) {
 		iput(inode);
 		return ERR_PTR(-ENOMEM);
 	}
-	result->d_flags |= DCACHE_NFSD_DISCONNECTED;
+	result->d_op = sb->s_root->d_op;
+	result->d_vfs_flags |= DCACHE_REFERENCED;
 	return result;
-
 		
 }
 
-int fat_dentry_to_fh(struct dentry *de, __u32 *fh, int *lenp, int needparent)
+int fat_encode_fh(struct dentry *de, __u32 *fh, int *lenp, int connectable)
 {
 	int len = *lenp;
 	struct inode *inode =  de->d_inode;
@@ -526,6 +528,43 @@ int fat_dentry_to_fh(struct dentry *de, __u32 *fh, int *lenp, int needparent)
 	return 3;
 }
 
+struct dentry *fat_get_parent(struct dentry *child)
+{
+	struct buffer_head *bh=NULL;
+	struct msdos_dir_entry *de = NULL;
+	struct dentry *parent = NULL;
+	int res;
+	int ino = 0;
+	struct inode *inode;
+
+	lock_kernel();
+	res = fat_scan(child->d_inode, MSDOS_DOTDOT, &bh, &de, &ino);
+
+	if (res < 0)
+		goto out;
+	inode = fat_build_inode(child->d_sb, de, ino, &res);
+	if (res)
+		goto out;
+	if (!inode)
+		res = -EACCES;
+	else {
+		parent = d_alloc_anon(inode);
+		if (!parent) {
+			iput(inode);
+			res = -ENOMEM;
+		}
+	}
+
+ out:
+	if(bh)
+		fat_brelse(child->d_sb, bh);
+	unlock_kernel();
+	if (res)
+		return ERR_PTR(res);
+	else
+		return parent;
+}
+
 static kmem_cache_t *fat_inode_cachep;
 
 static struct inode *fat_alloc_inode(struct super_block *sb)
@@ -580,8 +619,13 @@ static struct super_operations fat_sops = {
 	clear_inode:	fat_clear_inode,
 
 	read_inode:	make_bad_inode,
-	fh_to_dentry:	fat_fh_to_dentry,
-	dentry_to_fh:	fat_dentry_to_fh,
+};
+
+static struct export_operations fat_export_ops = {
+	decode_fh:	fat_decode_fh,
+	encode_fh:	fat_encode_fh,
+	get_dentry:	fat_get_dentry,
+	get_parent:	fat_get_parent,
 };
 
 /*
@@ -590,12 +634,13 @@ static struct super_operations fat_sops = {
 int fat_fill_super(struct super_block *sb, void *data, int silent,
 		   struct inode_operations *fs_dir_inode_ops, int isvfat)
 {
-	struct inode *root_inode;
+	struct inode *root_inode = NULL;
 	struct buffer_head *bh;
 	struct fat_boot_sector *b;
 	struct msdos_sb_info *sbi;
-	int logical_sector_size, fat_clusters, debug, cp;
+	int logical_sector_size, fat_clusters, debug, cp, first;
 	unsigned int total_sectors, rootdir_sectors;
+	unsigned char media;
 	long error = -EIO;
 	char buf[50];
 	int i;
@@ -614,6 +659,7 @@ int fat_fill_super(struct super_block *sb, void *data, int silent,
 
 	sb->s_magic = MSDOS_SUPER_MAGIC;
 	sb->s_op = &fat_sops;
+	sb->s_export_op = &fat_export_ops;
 	sbi->options.isvfat = isvfat;
 	sbi->dir_ops = fs_dir_inode_ops;
 	sbi->cvf_format = &default_cvf;
@@ -658,6 +704,13 @@ int fat_fill_super(struct super_block *sb, void *data, int silent,
 		brelse(bh);
 		goto out_invalid;
 	}
+	media = b->media;
+	if (!FAT_VALID_MEDIA(media)) {
+		if (!silent)
+			printk("FAT: invalid media value (0x%02x)\n", media);
+		brelse(bh);
+		goto out_invalid;
+	}
 	logical_sector_size =
 		CF_LE_W(get_unaligned((unsigned short *) &b->sector_size));
 	if (!logical_sector_size
@@ -780,6 +833,21 @@ int fat_fill_super(struct super_block *sb, void *data, int silent,
 
 	brelse(bh);
 
+	/* validity check of FAT */
+	first = __fat_access(sb, 0, -1);
+	if (first < 0) {
+		error = first;
+		goto out_fail;
+	}
+	if (FAT_FIRST_ENT(sb, media) != first) {
+		if (!silent) {
+			printk("FAT: invalid first entry of FAT "
+			       "(0x%x != 0x%x)\n",
+			       FAT_FIRST_ENT(sb, media), first);
+		}
+		goto out_invalid;
+	}
+
 	if (!strcmp(cvf_format, "none"))
 		i = -1;
 	else
@@ -819,18 +887,20 @@ int fat_fill_super(struct super_block *sb, void *data, int silent,
 			       sbi->nls_io->charset);
 	}
 
+	error = -ENOMEM;
 	root_inode = new_inode(sb);
 	if (!root_inode)
 		goto out_fail;
-
 	root_inode->i_ino = MSDOS_ROOT_INO;
 	root_inode->i_version = 0;
-	fat_read_root(root_inode);
+	error = fat_read_root(root_inode);
+	if (error < 0)
+		goto out_fail;
+	error = -ENOMEM;
 	insert_inode_hash(root_inode);
 	sb->s_root = d_alloc_root(root_inode);
 	if (!sb->s_root) {
 		printk("FAT: get root inode failed\n");
-		iput(root_inode);
 		goto out_fail;
 	}
 	if(i >= 0) {
@@ -843,6 +913,8 @@ out_invalid:
 	error = -EINVAL;
 
 out_fail:
+	if (root_inode)
+		iput(root_inode);
 	if (sbi->nls_io)
 		unload_nls(sbi->nls_io);
 	if (sbi->nls_disk)
@@ -872,17 +944,20 @@ int fat_statfs(struct super_block *sb,struct statfs *buf)
 		free = MSDOS_SB(sb)->free_clusters;
 	else {
 		free = 0;
-		for (nr = 2; nr < MSDOS_SB(sb)->clusters+2; nr++)
-			if (!fat_access(sb,nr,-1)) free++;
+		for (nr = 2; nr < MSDOS_SB(sb)->clusters + 2; nr++)
+			if (fat_access(sb, nr, -1) == FAT_ENT_FREE)
+				free++;
 		MSDOS_SB(sb)->free_clusters = free;
 	}
 	unlock_fat(sb);
+
 	buf->f_type = sb->s_magic;
 	buf->f_bsize = 1 << MSDOS_SB(sb)->cluster_bits;
 	buf->f_blocks = MSDOS_SB(sb)->clusters;
 	buf->f_bfree = free;
 	buf->f_bavail = free;
 	buf->f_namelen = MSDOS_SB(sb)->options.isvfat ? 260 : 12;
+
 	return 0;
 }
 
@@ -923,10 +998,11 @@ static struct address_space_operations fat_aops = {
 };
 
 /* doesn't deal with root inode */
-static void fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
+static int fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
 {
 	struct super_block *sb = inode->i_sb;
 	struct msdos_sb_info *sbi = MSDOS_SB(sb);
+	int error;
 
 	MSDOS_I(inode)->i_location = 0;
 	inode->i_uid = sbi->options.fs_uid;
@@ -947,16 +1023,19 @@ static void fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
 				(CF_LE_W(de->starthi) << 16);
 		}
 		MSDOS_I(inode)->i_logstart = MSDOS_I(inode)->i_start;
+		error = fat_calc_dir_size(inode);
+		if (error < 0)
+			return error;
+		MSDOS_I(inode)->mmu_private = inode->i_size;
+
 		inode->i_nlink = fat_subdirs(inode);
 		    /* includes .., compensating for "self" */
 #ifdef DEBUG
 		if (!inode->i_nlink) {
-			printk("directory %d: i_nlink == 0\n",inode->i_ino);
+			printk("directory %lu: i_nlink == 0\n",inode->i_ino);
 			inode->i_nlink = 1;
 		}
 #endif
-		fat_calc_dir_size(inode);
-		MSDOS_I(inode)->mmu_private = inode->i_size;
 	} else { /* not a directory */
 		inode->i_generation |= 1;
 		inode->i_mode = MSDOS_MKMODE(de->attr,
@@ -991,6 +1070,8 @@ static void fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
 		? date_dos2unix(CF_LE_W(de->ctime),CF_LE_W(de->cdate))
 		: inode->i_mtime;
 	MSDOS_I(inode)->i_ctime_ms = de->ctime_ms;
+
+	return 0;
 }
 
 void fat_write_inode(struct inode *inode, int wait)
