@@ -37,6 +37,7 @@
 
 #include <linux/blkdev.h>
 #include <linux/fs.h>
+#include <linux/buffer_head.h>
 #include <linux/kernel.h>
 #include <linux/slab.h>
 #include <linux/stat.h>
@@ -237,6 +238,7 @@ static struct file_system_type vxfs_fs_type = {
 	owner:		THIS_MODULE,
 	name:		"vxfs",
 	get_sb:		vxfs_get_sb,
+	kill_sb:	kill_block_super,
 	fs_flags:	FS_REQUIRES_DEV,
 };
 
