@@ -21,7 +21,6 @@
 #include <linux/string.h>
 #include <linux/stat.h>
 #include <linux/errno.h>
-#include <linux/locks.h>
 #include <linux/file.h>
 #include <linux/fcntl.h>
 #include <linux/slab.h>
@@ -795,8 +794,6 @@ static void __exit exit_ncp_fs(void)
 #endif
 }
 
-EXPORT_NO_SYMBOLS;
-
 module_init(init_ncp_fs)
 module_exit(exit_ncp_fs)
 MODULE_LICENSE("GPL");
