@@ -21,8 +21,8 @@
  * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <linux/locks.h>
 #include <linux/fs.h>
+#include <linux/buffer_head.h>
 
 #include "ntfs.h"
 
@@ -198,7 +198,7 @@ return_error:
 			flush_dcache_page(dp);
 			kunmap(dp);
 			SetPageUptodate(dp);
-			UnlockPage(dp);
+			unlock_page(dp);
 			if (di == xpage)
 				*xpage_done = 1;
 			else
@@ -491,7 +491,7 @@ int ntfs_file_read_compressed_block(struct page *page)
 		kfree(bhs);
 		kfree(pages);
 		SetPageError(page);
-		UnlockPage(page);
+		unlock_page(page);
 		ntfs_error(vol->sb, "Failed to allocate internal buffers.");
 		return -ENOMEM;
 	}
@@ -521,13 +521,13 @@ int ntfs_file_read_compressed_block(struct page *page)
 			 * in and/or dirty or we would be losing data or at
 			 * least wasting our time.
 			 */
-			if (!PageDirty(page) && (!Page_Uptodate(page) ||
+			if (!PageDirty(page) && (!PageUptodate(page) ||
 					PageError(page))) {
 				ClearPageError(page);
 				kmap(page);
 				continue;
 			}
-			UnlockPage(page);
+			unlock_page(page);
 			page_cache_release(page);
 			pages[i] = NULL;
 		}
@@ -668,7 +668,7 @@ retry_remap:
 				flush_dcache_page(page);
 				kunmap(page);
 				SetPageUptodate(page);
-				UnlockPage(page);
+				unlock_page(page);
 				if (cur_page == xpage)
 					xpage_done = 1;
 				else
@@ -738,7 +738,7 @@ retry_remap:
 				flush_dcache_page(page);
 				kunmap(page);
 				SetPageUptodate(page);
-				UnlockPage(page);
+				unlock_page(page);
 				if (cur2_page == xpage)
 					xpage_done = 1;
 				else
@@ -763,9 +763,10 @@ retry_remap:
 		 * ntfs_decompess.
 		 */
 		if (err) {
-			ntfs_error(vol->sb, "ntfs_decompress() failed with "
-					"error code %i. Skipping this "
-					"compression block.\n", -err);
+			ntfs_error(vol->sb, "ntfs_decompress() failed in inode "
+					"0x%Lx with error code %i. Skipping "
+					"this compression block.\n",
+					(unsigned long long)ni->mft_no, -err);
 			/* Release the unfinished pages. */
 			for (; prev_cur_page < cur_page; prev_cur_page++) {
 				page = pages[prev_cur_page];
@@ -775,7 +776,7 @@ retry_remap:
 						SetPageError(page);
 					flush_dcache_page(page);
 					kunmap(page);
-					UnlockPage(page);
+					unlock_page(page);
 					if (prev_cur_page != xpage)
 						page_cache_release(page);
 					pages[prev_cur_page] = NULL;
@@ -806,7 +807,7 @@ retry_remap:
 				SetPageError(page);
 			flush_dcache_page(page);
 			kunmap(page);
-			UnlockPage(page);
+			unlock_page(page);
 			if (cur_page != xpage)
 				page_cache_release(page);
 			pages[cur_page] = NULL;
@@ -853,7 +854,7 @@ err_out:
 				SetPageError(page);
 			flush_dcache_page(page);
 			kunmap(page);
-			UnlockPage(page);
+			unlock_page(page);
 			if (i != xpage)
 				page_cache_release(page);
 		}
