@@ -11,6 +11,8 @@
 
 #include <linux/smb.h>
 #include <linux/smb_fs_i.h>
+#include <linux/smb_fs_sb.h>
+#include <linux/fs.h>
 
 /*
  * ioctl commands
@@ -29,6 +31,11 @@
 #include <linux/smb_mount.h>
 #include <asm/unaligned.h>
 
+static inline struct smb_sb_info *SMB_SB(struct super_block *sb)
+{
+	return sb->u.generic_sbp;
+}
+
 static inline struct smb_inode_info *SMB_I(struct inode *inode)
 {
 	return list_entry(inode, struct smb_inode_info, vfs_inode);
