@@ -493,7 +493,7 @@ static int send_signal(int sig, struct siginfo *info, struct sigpending *signals
  * No need to set need_resched since signal event passing
  * goes through ->blocked
  */
-static inline void signal_wake_up(struct task_struct *t)
+inline void signal_wake_up(struct task_struct *t)
 {
 	set_tsk_thread_flag(t,TIF_SIGPENDING);
 
@@ -1110,7 +1110,8 @@ do_sigaction(int sig, const struct k_sigaction *act, struct k_sigaction *oact)
 		    || (k->sa.sa_handler == SIG_DFL
 			&& (sig == SIGCONT ||
 			    sig == SIGCHLD ||
-			    sig == SIGWINCH))) {
+			    sig == SIGWINCH ||
+			    sig == SIGURG))) {
 			spin_lock_irq(&current->sigmask_lock);
 			if (rm_sig_from_queue(sig, current))
 				recalc_sigpending();
